"use strict";
var bodyParser = require("body-parser");
var express = require("express");
var path_1 = require("path");
var DefaultCommands = require("./default-commands");
var Server = (function () {
    /**
     * Instance a new mock server
     *
     * @constructor
     * @param {string|number} port The port which the server will listen on
     * @param {function} initSession An initialiser for any custom session.  Will receive a session
     *   with only the `id` and `capabilities` fields filled in.
     * @param {string} basePath The basic path for selenium commands to be executed against.  Defaults
     *   to `/wd/hub`.
     */
    function Server(port, initSession, basePath) {
        if (initSession === void 0) { initSession = null; }
        if (basePath === void 0) { basePath = '/wd/hub'; }
        var _this = this;
        this.port = port;
        this.initSession = initSession;
        this.basePath = basePath;
        this.app = express();
        this.sessions = [];
        if (initSession) {
            // This is a fun hack yeah?
            this.sessions.push = function () {
                var sessions = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    sessions[_i] = arguments[_i];
                }
                sessions = sessions.map(initSession);
                return Array.prototype.push.apply(_this.sessions, sessions);
            };
        }
        this.app.use(bodyParser.json());
        this.addGlobalCommand(DefaultCommands.getStatus);
        this.addGlobalCommand(DefaultCommands.newSession);
        this.addGlobalCommand(DefaultCommands.getSessions);
        this.addGlobalCommand(DefaultCommands.deleteSession);
        this.addCommand(DefaultCommands.getSession);
        this.addSpecialRoute('GET', '/selenium-server/driver/', function (req, res) {
            if (req.query.cmd == 'shutDownSeleniumServer') {
                _this.stop();
            }
            else if (req.query.cmd == 'getLogMessages') {
                res.send('OK');
            }
        });
    }
    /**
     * Add a new command for the server to handle
     * @param {Command<T>} command The specification for what urls to handle and how to handle them
     */
    Server.prototype.addCommand = function (command) {
        this.addGlobalCommand(command._globalize());
    };
    /**
     * Add a new command for the server to handle
     * @param {GlobalCommand<T>} command The specification for what urls to handle/how to handle them
     */
    Server.prototype.addGlobalCommand = function (command) {
        var _this = this;
        var handle = function (req, res, next) {
            function fail(e) {
                res.json({
                    sessionId: req.params.sessionId,
                    status: e.status || e.code || 13,
                    value: e instanceof Error ? e.toString() : e.value != null ? e.value : e
                });
            }
            try {
                for (var name in req.body) {
                    req.params[name] = req.body[name];
                }
                for (var name in req.query) {
                    if (req.params[name] === undefined) {
                        req.params[name] = req.query[name];
                    }
                }
                Promise.resolve(command.exec(_this.sessions, req.params)).then(function (value) {
                    res.json({ sessionId: req.params.sessionId, status: 0, value: value });
                    next();
                }, fail);
            }
            catch (e) {
                fail(e);
            }
        };
        this.addSpecialRoute(command.method, path_1.posix.join(this.basePath, command.path), handle);
    };
    /**
     * Handle a ruote directly using the Express API.  Used for special cases
     *
     * @param {string}
     */
    Server.prototype.addSpecialRoute = function (method, path, handler) {
        switch (method) {
            case 'GET':
                this.app.get(path, handler);
                break;
            case 'POST':
                this.app.post(path, handler);
                break;
            case 'DELETE':
                this.app.delete(path, handler);
                break;
            case 'PUT':
                this.app.put(path, handler);
                break;
            default:
                throw new Error('Invalid method "' + method + '"');
        }
    };
    /**
     * Start the server
     */
    Server.prototype.start = function () {
        this.handle = this.app.listen(this.port);
    };
    /**
     * Close down the server
     */
    Server.prototype.stop = function () {
        this.handle.close();
    };
    return Server;
}());
exports.Server = Server;
//# sourceMappingURL=server.js.map